--[[

	===================================
	
	ROLES:
	
	"MAIN_TANK"
	"TANK"
	"TANK_BUFFER"
	"HEALER"
	"MANA_BATTERY"
	"HEAL_BUFFER"
	"MELEE_BUFFER"
	"RANGE_BUFFER"
	"SPELL_BUFFER"
	"MELEE_DPS"
	"RANGE_DPS"
	"SPELL_DPS"
	
	----------------------------------
	Note:
	-MAIN_TANK is currently always false for all members, it's there for future versions.
	-The "BUFFER" roles imply that that player has an aura or something that buffs people in their own group only.
	-TANK_BUFFER gets applied to warlocks with imp. imp, pallies with imp. devo aura, etc. It means a *better* tank buffer, but that doesn't mean that other members of that class shouldn't get added to tanking groups if no "tank buffers" are available. use "class" for that
	
	===================================
	
	
	
	This file contains a sample template, which exactly duplicates the functionality of the built-in auto-arrangement rules.
	If you would like to add your own templates, you can put them in this file, using the example one as a guide.
	If you would like to distribute templates, you can do so by sending people a copy of this file or by making a small addon which adds one or more entries into GrOM.userTemplates
	Of course, if you do that, you need to make sure that it adds them after GOM has loaded, for example by watching for VARIABLES_LOADED

	The basic layout of a template is a 3-index table. The first index is the name of the template. The second and third are both tables that specify a set of actions to follow:	
	
	Entries in both tables can be either a string, which gives a special command, or a table specifying the type of raid member you want to add to the current subgroup.
	
	The auto-arranger first assumes that each subgroup is empty and passes over them (up to the highest subgroup that is going to be used) and executes the actions in the first table for each group.
	It then passes over each subgroup again, executing the actions in the second table.
	
	This happens in one video frame, before the confirmation dialog. The code that actually moves people around starts after that.
	
	Valid strings:
		-go #
			-go to index # (must be higher than the current index)
		-fail #
			-if the most recent attempt to find a player to add (in the current subgroup, in the current pass) did not find anyone, go to index # (must be higher than the current index)
		-group #,#,#,#,...
			-only apply the next rule if the current subgroup is one of the arguments (the next rule must be a table, if it is a string then this is ignored)
		-spec ROLE #
			-if the current group already contains a member that has the role ROLE, go to index # (must be higher than the current index)
		-end
			-stop and go on to the next subgroup
			
	Valid Table Structure:
		{class, prefSpec, requirePref, ignoreSpec, loop}
		
		-class is a literal string. all caps, in english; or nil for any class
		-prefSpec is a ROLE string. you must specify something here, and it must be a valid role.
		-requirePref is a boolean. if it is true then only players matching prefSpec will be considered, otherwise they will just be preferred
		-ignoreSpec is a ROLE string. members with this spec will not be considered. Nil for none.
		-loop is a boolean. if it is true, the auto-arranger will repeat this step until it does not yield another match, or the group is full, or there are no more unassigned members to add
		
	General guidelines:
		-If your rules do not find a group for everyone in the raid, GOM will output an error and stop.
		-It's usually a good idea to put a general "add anyone else" at the very end
		-Obviously the "spec" command will always be false if you're using it in the first pass.
		-Don't put quotes around whatever ROLE you use with the "spec" command, they go around the whole thing like the other strings (i.e. "spec TANK_BUFFER 45")
		-If all of the remaining entries in a table are nil, you can omit them: {nil, "TANK", true} is the same as {nil, "TANK", true, nil, nil}
		
]]


--[[

GrOM.userTemplates["EXAMPLE_AKRYN"] = {
	"Example",
	{
	--we're creating a new group
	--if there is a tank available, add it and
		--if there is another tank available, add it too
		--if there is a tree druid that isn't a tank (not possible to be both atm but it can't hurt to be thorough), add it
		--if there is a pally that is not a tank and is a "tank buffer" available, add it; otherwise if there is a pally that isn't a tank available, add it
		--if there is a warlock that is a "tank buffer" available, add it; otherwise, if there is any other warlock available, add it			
		--stop
	--else if there is a healer available, add it and
		--if there is a mana battery, add it
		--if there is any free "heal buffer" add it
		--add as many other healers as there are free
		--stop
	--else if there is a melee dps available, add it and
		--if there is any melee buffer available, add it
		--find as many melee dps as are available, and add them
		--stop
	--else do the same as above for ranged and then spell dps
	
		[1] = {nil, "TANK", true},
		[2] = "fail 8",
		[3] = {nil, "TANK", true},
		[4] = {"DRUID", "TANK_BUFFER", true, "TANK"},
		[5] = {"PALADIN", "TANK_BUFFER", false, "TANK"},
		[6] = {"WARLOCK", "TANK_BUFFER", false},
		[7] = "end",
		[8] = {nil, "HEALER", true},
		[9] = "fail 14",
		[10] = {nil, "MANA_BATTERY", true},
		[11] = {nil, "HEAL_BUFFER", true},
		[12] = {nil, "HEALER", true, nil, true},
		[13] = "end",
		[14] = {nil, "MELEE_DPS", true},
		[15] = "fail 19",
		[16] = {nil, "MELEE_BUFFER", true},
		[17] = {nil, "MELEE_DPS", true, nil, true},
		[18] = "end",
		[19] = {nil, "RANGE_DPS", true},
		[20] = "fail 24",
		[21] = {nil, "RANGE_BUFFER", true},
		[22] = {nil, "RANGE_DPS", true, nil, true},
		[23] = "end",
		[24] = {nil, "SPELL_DPS", true}, --no need for a "fail" here because if this fails there are no tanks, healers or dps left to add, but there are still unassigned members, which would mean there's something really wrong :P
		[25] = {nil, "SPELL_BUFFER", true},
		[26] = {nil, "SPELL_DPS", true, nil, true}
		
	},
	{
	--we're filling in empty space
	--if there is a healer anywhere in the group
		--find any available heal buffers or mana batteries and add them
	--if there is a tank anywhere in the group
		--find any available tanks or tank buffers and add them
	--if there are any melee buffers in the group, add any available melee dps
	--if there are any ranged buffers in the group, add any available ranged dps and then melee dps
	--fill in the rest, starting with spell dps
	
		[1] = "spec HEALER 6",
		[2] = "spec TANK 9",
		[3] = "spec MELEE_BUFFER 12",
		[4] = "spec RANGE_BUFFER 14",
		[5] = "go 16",
		[6] = {nil, "HEAL_BUFFER", true, nil, true},
		[7] = {nil, "MANA_BATTERY", true, nil, true},
		[8] = "go 16",
		[9] = {nil, "TANK_BUFFER", true, nil, true},
		[10] = {nil, "TANK", true, nil, true},
		[11] = "go 16",
		[12] = {nil, "MELEE_DPS", true, nil, true},
		[13] = "go 16",
		[14] = {nil, "RANGE_DPS", true, nil, true},
		[15] = {nil, "MELEE_DPS", true, nil, true},
		[16] = {nil, "SPELL_DPS", false, nil, true} --will add anything left (starting with casters) until the group is full or there's nothing else to add
		
	}}
	
]]


GrOM.userTemplates["Alterac_Valley_AKRYN"] = {
	"Alterac Valley",
	
	{
		--set up each group with one healer and one of each class (ignoring other healers) until it's full
	
		[1] = {nil, "HEALER", true}, --add one healer to each group on the first pass
		[2] = {"ROGUE", "HEALER", false}, --healer because i don't want any pref spec, and none of these classes are likely to be healers
		[3] = {"WARRIOR", "HEALER", false},
		[4] = {"WARLOCK", "HEALER", false},
		[5] = {"DRUID", "TANK", false, "HEALER"}, --feral druids, or oomkin if none
		[6] = {"MAGE", "HEALER", false},
		[7] = {"HUNTER", "HEALER", false},
		[8] = {"PRIEST", "SPELL_DPS", true}, --spriests
		[9] = {"SHAMAN", "HEALER", false, "HEALER"}, --pref = ignore will give you anything that is not ignore without bias if requirepref is false
		[10] = {"PALADIN", "HEALER", false, "HEALER"}
	},
	{
		[1] = "spec HEALER 6",
		[2] = "spec TANK 9",
		[3] = "spec MELEE_BUFFER 12",
		[4] = "spec RANGE_BUFFER 14",
		[5] = "go 16",
		[6] = {nil, "HEAL_BUFFER", true, nil, true},
		[7] = {nil, "MANA_BATTERY", true, nil, true},
		[8] = "go 16",
		[9] = {nil, "TANK_BUFFER", true, nil, true},
		[10] = {nil, "TANK", true, nil, true},
		[11] = "go 16",
		[12] = {nil, "MELEE_DPS", true, nil, true},
		[13] = "go 16",
		[14] = {nil, "RANGE_DPS", true, nil, true},
		[15] = {nil, "MELEE_DPS", true, nil, true},
		[16] = {nil, "SPELL_DPS", false, nil, true}
	}
}